@@ -184,12 +184,20 @@ class NaverKinCrawler(BaseCrawler):
     
     def default_breaker(self, response):
         return False
+    
+    def default_suspender(self, response, suspended, context):
+        print('suspended:', suspended)
+        return 10
+        
 
 
+def event_listener(module, event, count):
+    print(module, event, count)
+    return True
 
 def test_naverkin(keywords, page_range):
     nkc = NaverKinCrawler(keywords, page_range,
         settings=settings
     )
-    nkc.crawl()
+    nkc.crawl(crawl_listener=event_listener)
     return nkc
